<?php

class ThirdParty {
    const URl = 'https://api.weixin.qq.com/cgi-bin/component/';

    public $component_appid = '';
    public $obj;


    public function __construct($obj)
    {
        $this->obj = $obj;
        $this->component_appid = getConfig('component_appid', 'wx86d3c80372c2c939');
        $this->component_appsecret = getConfig('component_appsecret', '9478b4ce87316c3f038f60c48645b1e0');
    }




    /**
     * 预授权码
     */
    public function pre_auth_code()
    {
        $pre_auth_code = cache_load($this->component_appid . 'pre_auth_code');
        if (!$pre_auth_code) {
            $component_access_token = $this->component_access_token();

            $data = array(
                'component_appid' => $this->component_appid
            );
            $url = 'https://api.weixin.qq.com/cgi-bin/component/api_create_preauthcode?component_access_token=' . $component_access_token;
            $result = $this->http_post($url, $data, true);
            if (isset($result['pre_auth_code'])) {
                $pre_auth_code = $result['pre_auth_code'];
                cache_write($this->component_appid . 'pre_auth_code', $result['pre_auth_code'], $result['expires_in'] - 100);
            }
        }
        return $pre_auth_code;
    }


    /**
     * 获取令牌
     */
    public function component_access_token()
    {
        $component_access_token = cache_load($this->component_appid . 'component_access_token');
        if (!$component_access_token) {

            $url = 'https://api.weixin.qq.com/cgi-bin/component/api_component_token';
            $data = array(
                'component_appid' => $this->component_appid,
                'component_appsecret' => $this->component_appsecret,
                'component_verify_ticket' => cache_load($this->component_appid . 'component_verify_ticket')
            );

            $result = $this->http_post($url, $data, true);

            if (isset($result['component_access_token'])) {
                $component_access_token = $result['component_access_token'];
                cache_write($this->component_appid . 'component_access_token', $result['component_access_token'], $result['expires_in'] - 100);
            }
        }
        return $component_access_token;
    }


    /**
     * 获取绑定链接
     */
    public function bindcomponent($parent_id = 0)
    {
        $token = $this->obj->get('token');
        if ($parent_id > 0) {
            $token .= ('_' . $parent_id);
        }
        $pre_auth_code = $this->pre_auth_code();
        $redirect_uri = urlencode($this->obj->w['siteroot'] . "addons/bh_vote/notify.php?token=" . $token);

        return "https://mp.weixin.qq.com/safe/bindcomponent?action=bindcomponent&no_scan=1&component_appid={$this->component_appid}&pre_auth_code={$pre_auth_code}&redirect_uri={$redirect_uri}&auth_type=1#wechat_redirect";
    }


    /**
     * 获取授权码
     */
    public function api_query_auth($authorization_code, $test = false, $p_id = 0)
    {
        $component_access_token = $this->component_access_token();
        $url = 'https://api.weixin.qq.com/cgi-bin/component/api_query_auth?component_access_token=' . $component_access_token;

        $data = array(
            'component_appid' => $this->component_appid,
            'authorization_code' => $authorization_code
        );
        $result = $this->http_post($url, $data, true);

        if ($test) {
            return $result['authorization_info']['authorizer_access_token'];
        }

        if (isset($result['authorization_info'])) {
            $third_party = p_get('third_party', array('member_id' => $this->obj->member['id'], 'authorizer_appid' => $result['authorization_info']['authorizer_appid']));

            $authorizer_info = $this->api_get_authorizer_info($result['authorization_info']['authorizer_appid']);

            $data = array(
                'member_id' => $p_id > 0 ? $p_id : $this->obj->member['id'],
                'authorizer_appid' => $result['authorization_info']['authorizer_appid'],
                'authorizer_access_token' => $result['authorization_info']['authorizer_access_token'],
                'authorizer_refresh_token' => $result['authorization_info']['authorizer_refresh_token'],
                'expires_in' => $result['authorization_info']['expires_in'] + time() - 100,
                'authorizer_info' => json_encode($authorizer_info),
                'qrcode_url' => $this->imageLocal($authorizer_info['authorizer_info']['qrcode_url'], $result['authorization_info']['authorizer_appid'] . '.png'),
                'source' => $p_id > 0 ? 2 : 1
            );

            if (empty($third_party)) {

                p_insert('third_party', $data);

            } else {
                p_update('third_party', $data);

            }
            $appid = pdo_getcolumn('account_wxapp', array('uniacid' => 2), 'key');
            $this->wxamplinkget($appid, $result['authorization_info']['authorizer_access_token']);
            return true;

        } else {
            return false;
        }
    }


    /**
     * 是否已经关联小程序
     */
    public function wxamplinkget($appid, $access_token)
    {
        $url = 'https://api.weixin.qq.com/cgi-bin/wxopen/wxamplinkget?access_token=' . $access_token;
        $res = $this->http_post($url);

        if (isset($res['errmsg']) && $res['errcode'] == 0) {
            foreach ($res['wxopens']['items'] as $value) {
                if ($value['appid'] == $appid && $value['status'] == 1) {
                    return true;
                }
            }
        }

        $this->wxamplink($appid, $access_token);
        return false;
    }



    /**
     * 发送关联
     */
    public function wxamplink($appid, $access_token = '')
    {
        //$access_token = $access_token ? : $this->
        $url = 'https://api.weixin.qq.com/cgi-bin/wxopen/wxamplink?access_token=' . $access_token;
        $data = array(
            'appid' => $appid,
            'notify_users' => 1,
            'show_profile' => 1
        );
        return $this->http_post($url, $data, true);
    }







    /**
     * 获取授权方的帐号基本信息
     * @param $authorizer_appid
     * @return string
     */
    public function api_get_authorizer_info($authorizer_appid)
    {
        $component_access_token = $this->component_access_token();
        $url = 'https://api.weixin.qq.com/cgi-bin/component/api_get_authorizer_info?component_access_token=' . $component_access_token;

        $data = array(
            'component_appid' => $this->component_appid,
            'authorizer_appid' => $authorizer_appid
        );

        return $this->http_post($url, $data, true);
    }


    /**
     * 图片保存到本地
     * @param $img_url
     * @return string
     */
    public function imageLocal($img_url, $path_name)
    {
        $qrpath = createFile($this->obj, 'ThirdParty') . $path_name;

        $ch = curl_init();
        curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($ch, CURLOPT_URL, $img_url);
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($ch, CURLOPT_HEADER, 0);
        $ret = curl_exec($ch);

        curl_close($ch);
        file_put_contents($qrpath, $ret);

        return 'xcx' . $this->obj->w['uniacid'] .'ThirdParty/' . $path_name;
    }



    public function reply($data, $txt)
    {
        if ($data['MsgType'] != 'text') {
            return false;
        }

        $content = '<xml>
  <ToUserName><![CDATA[%s]]></ToUserName>
  <FromUserName><![CDATA[%s]]></FromUserName>
  <CreateTime>%s</CreateTime>
  <MsgType><![CDATA[%s]]></MsgType>
  <Content><![CDATA[%s]]></Content>
</xml>';

        $toUser = $data['FromUserName'];
        $fromUser = $data['ToUserName'];
        $CreateTime = $data['CreateTime'];
        $MsgType = 'text';
        return sprintf($content, $toUser, $fromUser, $CreateTime, $MsgType, $txt);

    }


    /**
     * 刷新更新令牌
     */
    public function api_authorizer_token($authorizer_refresh_token, $appid)
    {
        $component_access_token = $this->component_access_token();
        $url = 'https://api.weixin.qq.com/cgi-bin/component/api_authorizer_token?component_access_token=' . $component_access_token;

        $data = array(
            'component_appid' => $this->component_appid,
            'authorizer_appid' => $appid,
            'authorizer_refresh_token' => $authorizer_refresh_token
        );
        $res = $this->http_post($url, $data, true);

        if (isset($res['authorizer_access_token'])) {
            $res['expires_in'] = $res['expires_in'] + time() - 100;
            p_update('third_party', $res, array('authorizer_appid' => $appid));

            return $res['authorizer_access_token'];
        }
        return false;

    }


    /**
     * 获取接口调用令牌
     * @param $appid
     * @return bool
     */
    public function authorizer_access_token($appid)
    {
        $third_party = p_get('third_party', array('authorizer_appid' => $appid));
        if (empty($third_party)) {
            return false;
        }

        if ($third_party['expires_in'] > time()) {
            return $third_party['authorizer_access_token'];
        }

        return $this->api_authorizer_token($third_party['authorizer_refresh_token'], $appid);
    }


    /**
     * 发送消息
     * @param $data
     * @param string $appid
     * @param string $access_token
     * @param string $content
     * @return bool|string
     */
    public function send($data, $access_token = '', $appid = '', $content = '')
    {
        if (empty($access_token)) {
            $appid = str_replace( '/', '', $appid);
            $access_token = $this->authorizer_access_token($appid);
            //file_put_contents(IA_ROOT . '/101.txt', 'access_token:'  . var_export($access_token, true), FILE_APPEND);
            if (!$access_token) {
                return false;
            }
        }

        $url = 'https://api.weixin.qq.com/cgi-bin/message/custom/send?access_token=' . $access_token;

        if (empty($content)) {

            $keyword_reply = p_get('luck_keyword_reply', array('appid' => $appid, 'content' => $data['Content']));

            //file_put_contents(IA_ROOT . '/101.txt', 'keyword_reply:'  . var_export($keyword_reply, true), FILE_APPEND);
            if (empty($keyword_reply)) {
                return false;
            }
            global $_W;

            $content = stripcslashes("<a  data-miniprogram-appid='{$_W['uniaccount']['key']}' data-miniprogram-path='bh_vote/pages/userGame/userGame?id={$keyword_reply['luck_id']}'>点我参与抽奖</a>");

        }

        $data = array(
            'touser' => $data['FromUserName'],
            'msgtype' => 'text',
            'text' => array(
                'content' => $content
            )
        );
        $data = str_replace('\/', '/', json_encode($data, JSON_UNESCAPED_UNICODE));
        $data = str_replace('\\', '', $data);

        //file_put_contents(IA_ROOT . '/101.txt', 'content:'  . var_export($data, true), FILE_APPEND);

        return $this->http_post($url, $data);
    }



    /**
     * POST 请求
     * @param string $url
     * @param array $param
     * @param boolean $post_file 是否文件上传
     * @return string content
     */
    private function http_post($url, $param = '', $jsonpost = false, $post_file = false) {
        $oCurl = curl_init();
        if (stripos($url, "https://") !== FALSE) {
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($oCurl, CURLOPT_SSLVERSION, 1); //CURL_SSLVERSION_TLSv1
        }
        if ($jsonpost) {
            $strPOST =  str_replace('\/', '/', json_encode($param, JSON_UNESCAPED_UNICODE));
            file_put_contents(IA_ROOT . '/101.txt', 'content:'  . var_export($strPOST, true), FILE_APPEND);
        } elseif (is_string($param) || $post_file) {
            $strPOST = $param;
        } else {
            $aPOST = array();
            foreach ($param as $key => $val) {
                $aPOST[] = $key . "=" . urlencode($val);
            }
            $strPOST = join("&", $aPOST);
        }
        curl_setopt($oCurl, CURLOPT_URL, $url);
        curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($oCurl, CURLOPT_POST, true);
        curl_setopt($oCurl, CURLOPT_POSTFIELDS, $strPOST);
        $sContent = curl_exec($oCurl);
        $aStatus = curl_getinfo($oCurl);
        curl_close($oCurl);
        if (intval($aStatus["http_code"]) == 200) {
            return json_decode($sContent, true);
        } else {
            return false;
        }
    }
}