<?php


class WxAccount{

    public $token = '';
    public $error = '';
    public $appid = '';
    public $secret = '';

    const API_URL_PREFIX = 'https://api.weixin.qq.com/cgi-bin/';


    const CONTENT = '<img src="{img}"><br/>长按识别上方二维码, 回复<text color="color:#FF0000">{keyword}</text>, 参与抽奖';

    public function __construct()
    {
        $this->appid = getConfig('gzh_appid', 'wx85a79b7f24068ce7');
        $this->secret = getConfig('gzh_secret', 'a62713b800f79cf2a0ff1f72cff4b706');
    }

    /**
     * 获取token
     */
    public function getAccessToken() {
        $key = 'WxAccount' . $this->appid;

        $token = cache_load($key);
        if ($token) {
            return $token;
        }
        if (empty($this->appid) || empty($this->secret)) {
            return error('-1', '未填写公众号的 appid 或 appsecret！');
        }
        $url = self::API_URL_PREFIX . 'token';

        $data = [
            'grant_type' => 'client_credential',
            'appid' => $this->appid,
            'secret' => $this->secret
        ];
        $data = $this->http_get($url, $data);

        $token = $this->result($data);

        if (!$token) {
            return false;
        }

        cache_write($key, $token['access_token'], $token['expires_in'] - 200);

        return $token['access_token'];
    }


    /**
     * 获取上传图文消息内的图片url
     */
    public function uploadimg($filePath)
    {
        $token = $this->getAccessToken();
        if (!$token) {
            return false;
        }
        $url = self::API_URL_PREFIX . "media/uploadimg?access_token={$token}";

        $obj = new \CURLFile(realpath($filePath));
        $ext = explode('.', $filePath);
        $extension = end($ext);
        $obj->setMimeType("image/{$extension}");

        $data = ['media' => $obj];
        $res = $this->http_post($url, $data, false, true);

        return $this->result($res, 'url');
    }


    /**
     * 删除永久素材
     */
    public function delMaterial($media_id)
    {
        $token = $this->getAccessToken();
        if (!$token) {
            return false;
        }
        $url = self::API_URL_PREFIX . "material/del_material?access_token={$token}";

        $data = ['media_id' => $media_id];
        $this->http_post($url, $data);
        return true;
    }

    /**
     * 获取永久素材url
     */
    public function getMaterialUrl($media_id)
    {
        $token = $this->getAccessToken();
        if (!$token) {
            return false;
        }
        $url = self::API_URL_PREFIX . "material/get_material?access_token={$token}";

        $data = ['media_id' => $media_id];
        $res = $this->http_post($url, $data, true);

        if (!$res = $this->result($res, 'news_item')) {
            return false;
        }

        return $res[0]['url'];
    }


    /**
     * 新增永久图文素材
     */
    public function addMaterialNews($title, $img_path, $keyword)
    {
        $token = $this->getAccessToken();
        if (!$token) {
            return false;
        }

        if (!$img = $this->uploadimg($img_path)) {
            return false;
        }
        $content = str_replace('{img}', $img, self::CONTENT);
        $content = str_replace('{keyword}', $keyword, $content);

        $data = [
            'articles' => [[
                'title' => $title,
                'thumb_media_id' => getConfig('thumb_media_id'),
                'author' => getConfig('xcx_title'),
                'show_cover_pic' => 0,
                'content' => $content,
                'content_source_url' => '',
                'digest' => '',
                'need_open_comment' => 0,
                'only_fans_can_comment' => 0
              ]
            ]
        ];

        $url = self::API_URL_PREFIX . "material/add_news?access_token={$token}";

        //$data = stripslashes(urldecode(json_encode($data, JSON_UNESCAPED_UNICODE)));

        $res = $this->http_post( $url, $data, true);

        return $this->result($res, 'media_id');
    }


    /**
     * 编辑素材
     * @param $data
     * @return array|bool|mixed
     */
    public function editMaterialNews($media_id, $title, $thumb_media_id = '', $author = '') {
        $token = $this->getAccessToken();
        if (is_error($token)) {
            return $token;
        }
        $data = [
            'media_id' => $media_id,
            'index' => 0,
            'title' => $title,
            'thumb_media_id' => $thumb_media_id ?: \think\Config::get('thumb_media_id'),
            'author' => $author ?: \think\Config::get('wx_axxount_author'),
            'digest' => '',
            'show_cover_pic' => 0,
            'content' => '',
            'content_source_url' => ''

        ];

        $url = "https://api.weixin.qq.com/cgi-bin/material/update_news?access_token={$token}";
        $response = $this->requestApi($url, stripslashes(addslashes(json_encode($data, JSON_UNESCAPED_UNICODE))));
        if (is_error($response)) {
            return $response;
        }

        return true;
    }



    /**
     * POST 请求
     * @param string $url
     * @param array $param
     * @param boolean $post_file 是否文件上传
     * @return string content
     */
    private function http_post($url, $param, $jsonpost = false, $post_file = false) {
        $oCurl = curl_init();
        if (stripos($url, "https://") !== FALSE) {
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, false);
            curl_setopt($oCurl, CURLOPT_SSLVERSION, 1); //CURL_SSLVERSION_TLSv1
        }
        if ($jsonpost) {
            $strPOST = str_replace('\/', '/', json_encode($param, JSON_UNESCAPED_UNICODE));
        } elseif (is_string($param) || $post_file) {
            $strPOST = $param;
        } else {
            $aPOST = array();
            foreach ($param as $key => $val) {
                $aPOST[] = $key . "=" . urlencode($val);
            }
            $strPOST = join("&", $aPOST);
        }
        curl_setopt($oCurl, CURLOPT_URL, $url);
        curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, 1);
        curl_setopt($oCurl, CURLOPT_POST, true);
        curl_setopt($oCurl, CURLOPT_POSTFIELDS, $strPOST);
        $sContent = curl_exec($oCurl);
        $aStatus = curl_getinfo($oCurl);
        curl_close($oCurl);
        if (intval($aStatus["http_code"]) == 200) {
            return $sContent;
        } else {
            return false;
        }
    }


    /**
     * @param $url
     * @param $data
     * @return bool|mixed
     */
    private function http_get($url, $data) {
        $oCurl = curl_init();
        if (stripos($url, "https://") !== FALSE) {
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYPEER, FALSE);
            curl_setopt($oCurl, CURLOPT_SSL_VERIFYHOST, FALSE);
            curl_setopt($oCurl, CURLOPT_SSLVERSION, 1); //CURL_SSLVERSION_TLSv1
        }
        $param = array();
        foreach ($data as $key => $val) {
            if (is_array($val)) {
                foreach ($val as $v) {
                    $param[] = $key . '[]=' . urlencode($v);
                }
            } else {
                $param[] = $key . "=" . urlencode($val);
            }
        }
        $sparam = join("&", $param);
        $this->geturl = $url . '?' . $sparam;
        curl_setopt($oCurl, CURLOPT_URL, $url . '?' . $sparam);
        curl_setopt($oCurl, CURLOPT_RETURNTRANSFER, 1);
        $sContent = curl_exec($oCurl);
        $aStatus = curl_getinfo($oCurl);
        curl_close($oCurl);
        if (intval($aStatus["http_code"]) == 200 || !empty($sContent)) {
            return str_replace(array("\n", "\r"), '', $sContent);
        } else {
            $this->error = intval($aStatus["http_code"]) . 'curl_error' . $this->geturl;
            return false;
        }
    }


    public function result($data, $return = '')
    {
        $data = json_decode($data, true);
        if (!empty($data['errcode'])) {
            $this->error = "访问公众平台接口失败, 错误: {$data['errmsg']},错误详情：{$data['errcode']}";
            return false;
        }
        return $return ? $data[$return] : $data;
    }

}