<?php
defined('IN_IA') or exit('Access Denied');

class shopModel{

    public $signin_days = array(
        1 => array(
            'key' => 'signin_one_day',
            'default' => 2
        ),
        2 => array(
            'key' => 'signin_two_day',
            'default' => 2
        ),
        3 => array(
            'key' => 'signin_three_day',
            'default' => 2
        ),
        4 => array(
            'key' => 'signin_four_day',
            'default' => 2
        ),
        5 => array('key' => 'signin_five_day',  'default' => 2),
        6 => array('key' => 'signin_six_day',  'default' => 2),
        7 => array('key' => 'signin_seven_day',  'default' => 2)
    );


    public $error = '';


    /**
     * @param $member_id
     * @param string $same_day
     * @return bool
     */
    public function get_step($member_id, $same_day = '')
    {
        if (empty($same_day)) {
            $same_day = strtotime(date('Y-m-d'));
        }
        return p_get('shop_step', array('member_id' => $member_id, 'same_day' => $same_day));
    }


    /**
     * @param $goods_id
     * @return int
     */
    public function get_inventory($goods_id)
    {
        $goods = p_get('shop_goods', array('id' => $goods_id));
        if ($goods['inventory_type'] == 1) {
            $today = strtotime(date('Y-m-d'));
            $sql = "SELECT COUNT(*) FROM " . prefix('shop_order') . " WHERE goods_id = {$goods_id} AND status < 4 AND created > $today";
            $sell_out = p_fetchcolumn($sql);
        } else {
            $sql = "SELECT COUNT(*) FROM " . prefix('shop_order') . " WHERE goods_id = {$goods_id} AND status < 4";
            $sell_out = p_fetchcolumn($sql);
        }

        return $goods['inventory'] - $sell_out ? : 0;
    }


    /**
     * @param $member_id
     * @param $goods_id
     * @return bool
     */
    public function exchange($member_id, $goods_id)
    {
        $goods = p_get('shop_goods', array('id' => $goods_id));
        if ($goods['currency'] > 0) {
            if (!$this->change_currency($member_id, $goods['currency'], 1)) {
                return false;
            }
        }

        $insert = array(
            'goods_id' => $goods_id,
            'member_id' => $member_id,
            'exchange_type' => $goods['exchange_type'],
            'shop_id' => $goods['shop_id'],
            'salesman_id' => $goods['salesman_id'],
            'created' => time()
        );
        if ($address = p_get('address', array('member_id' => $member_id))) {
            $insert['name'] = $address['name'];
            $insert['phone'] = $address['phone'];
            $insert['address'] = $address['address'];
        }

        if ($goods['money'] > 0 && $goods['exchange_type'] == 2) {
            $insert['money'] = $goods['money'];
        }
        if ($goods['free'] > 0 && $goods['is_free'] == 2) {
            $insert['free'] = $goods['free'];
        }

        if (!p_insert('shop_order', $insert)) {
            $this->error = '失败';
            return false;
        }

        if ($goods['exchange_type'] == 2 && $goods['money'] > 0) {
            if ($goods['shop_id'] > 0) {
                $insert_id = pdo_insertid();
                if (!$this->change_shop_money($goods['shop_id'], $goods['money'], 1, $insert_id)) {
                    return false;
                }
                if ($goods['salesman_id'] > 0) {
                    $proportion = getConfig('salesman_proportion', 0);
                    if ($proportion > 0) {
                        $money = round(($goods['money'] * $proportion) / 100, 2);
                        if ($money > 0 && !$this->change_salesman_money($goods['salesman_id'], $money, 1, $insert_id)) {
                            return false;
                        }

                    }
                }
            }
        }

        return true;
    }


    /**
     * @param $shop_id
     * @param $money
     * @param $type
     * @param $relation_id
     * @return bool
     */
    public function change_shop_money($shop_id, $money, $type, $relation_id)
    {
        if ($money <= 0) {
            return true;
        }
        $shop = p_get('shop', array('id' => $shop_id));
        if (in_array($type, array(1, 3))) {
            $new_money = $shop['money'] + $money;
        } else {
            if ($money > $shop['money']) {
                $this->error = '您的余额不足';
                return false;
            }
            $new_money = $shop['money'] - $money;
        }

        if (!p_update('shop', array('money' => $new_money), array('id' => $shop_id))) {
            $this->error = '商家余额变更失败';
            return false;
        }

        $insert = array(
            'shop_id' => $shop_id,
            'relation_id' => $relation_id,
            'money' => $money,
            'salesman_id' => $shop['salesman_id'],
            'type' => $type,
            'created' => time()
        );
        if (!p_insert('shop_money', $insert)) {
            $this->error = '金额日志变更失败';
            return false;
        }
        return true;
    }



    /**
     * @param $salesman_id
     * @param $money
     * @param $type
     * @param $relation_id
     * @return bool
     */
    public function change_salesman_money($salesman_id, $money, $type, $relation_id)
    {
        if ($money <= 0) {
            return true;
        }
        $salesman = p_get('salesman', array('id' => $salesman_id));
        if (in_array($type, array(1, 3))) {
            $new_money = $salesman['money'] + $money;
        } else {
            if ($money > $salesman['money']) {
                $this->error = '您的余额不足';
                return false;
            }
            $new_money = $salesman['money'] - $money;
        }

        if (!p_update('salesman', array('money' => $new_money), array('id' => $salesman_id))) {
            $this->error = '业务员余额变更失败';
            return false;
        }

        $insert = array(
            'salesman_id' => $salesman_id,
            'relation_id' => $relation_id,
            'money' => $money,
            'type' => $type,
            'created' => time()
        );
        if (!p_insert('salesman_money', $insert)) {
            $this->error = '业务员金额日志变更失败';
            return false;
        }
        return true;
    }



    /**
     * @param $member_id
     * @param $currency
     * @param int $type
     * @param string $remarks
     * @return bool
     */
    public function change_currency($member_id, $currency, $type = 1, $remarks = '')
    {
        if ($currency <= 0) {
            return true;
        }
        $member = p_get('memberinfo', array('id' => $member_id));
        if (in_array($type, array(2, 5, 7, 8, 9, 10))) {
            $new_currency = $member['currency'] + $currency;
        } else {
            if ($currency > $member['currency']) {
                $this->error = '您的' . getConfig('currency_name', '活力币') . '不足';
                return false;
            }
            $new_currency = $member['currency'] - $currency;
        }
        if (!p_update('memberinfo', array('currency' => $new_currency), array('id' => $member_id))) {
            $this->error = getConfig('currency_name', '活力币') . '变更失败';
            return false;
        }

        $types = array(
            1 => '兑换商品',
            2 => '步数转化',
            3 => '参加夺宝',
            4 => '参加大转盘',
            5 => '大转盘奖品',
            6 => '参加挑战赛',
            7 => '挑战赛成功',
            8 => '绑定手机号',
            9 => '授权用户信息',
            10 => '邀请新用户'
        );
        if (empty($remarks)) {
            $remarks = $types[$type];
        }

        $data = array(
            'member_id' => $member_id,
            'type' => $type,
            'number' => $currency,
            'remarks' => $remarks,
            'created' => time()
        );
        if (!p_insert('currency_log', $data)) {
            $this->error = getConfig('currency_name', '活力币') . '变更失败';
            return false;
        }
        return true;
    }


    /**
     * @param $member_id
     * @param $step
     * @return bool
     */
    public function update_step($member_id, $step)
    {
        if ($step < 1) {
            return true;
        }

        $today = strtotime(date('Y-m-d'));
        $today_step = p_get('shop_step', array('member_id' => $member_id, 'same_day' => $today));

        $effective_step = getConfig('effective_step', 1000);

        if (empty($today_step)) {

            $effective = $step > $effective_step ? $effective_step : $step;
            $insert = array(
                'same_day' => $today,
                'own_step' => $step,
                'member_id' => $member_id,
                'own_step_exchange' => $effective,
                'created' => time()
            );

            if (!p_insert('shop_step', $insert)) {
                $this->error = '步数更新失败';
                return false;
            }

        } else {
            if ($step <= $today_step['own_step']) {
                return true;
            }

            $surplus = $effective_step - $today_step['own_step_exchange'];
            $surplus = $surplus > 0 ? $surplus : 0;

            $add = $step - $today_step['own_step'];

            $effective = $surplus > $add ? $add : $surplus;

            $update = array(
                'own_step' => $step,
                'own_step_exchange' => $today_step['own_step_exchange'] + $effective
            );

            if (!p_update('shop_step', $update, array('id' => $today_step['id']))) {
                $this->error = '步数更新失败';
                return false;
            }
        }

        if ($effective > 0 && !$this->step_log($member_id, $effective, 1, $today)) {
            return false;
        }
        $this->update_superior($member_id, $effective);

        return true;
    }


    /**
     * @param $member_id
     * @param $step
     * @param int $type
     * @param string $same_day
     * @return bool
     */
    public function step_log($member_id, $step, $type = 1, $same_day = '')
    {
        if (empty($same_day)) {
            $same_day = strtotime(date('Y-m-d'));
        }
        $insert = array(
            'member_id' => $member_id,
            'same_day' => $same_day,
            'type' => $type,
            'step' => $step,
            'created' => time()
        );
        if (!p_insert('shop_step_log', $insert)) {
            $this->error = '更新失败';
            return false;
        }
        return true;
    }


    /**
     * 更新上级步数
     * @param $member_id
     * @param $step
     * @return bool
     */
    public function update_superior($member_id, $step)
    {
        if ($step < 1) {
            return true;
        }
        $superior_proportion = getConfig('superior_proportion', 0);
        $step = floor($step * $superior_proportion / 100);

        if ($superior_proportion < 1 || $step < 1) {
            return true;
        }
        $m = p_get('memberinfo', array('id' => $member_id));


        if ((empty($m['nickname']) && empty($m['headimgurl'])) || empty($m['parent_id'])) {
            return true;
        }

        /*$today = strtotime(date('Y-m-d'));

        $today_step = p_get('shop_step', array('member_id' => $m['parent_id'], 'same_day' => $today));
        if ($today_step) {
            $insert = array(
                'same_day' => $today,
                'member_id' => $m['parent_id'],
                'created' => time()
            );
            if (!p_insert('shop_step', $insert)) {
                $this->error = '更新失败';
                return false;
            }
        } else {

        }*/
        return $this->step_log($m['parent_id'], $step, 2);
    }


    /**
     * @param $id
     * @param $member_id
     * @return bool
     */
    public function receive_step($id, $member_id)
    {
        $today = strtotime(date('Y-m-d'));
        $where = array(
            'id' => $id,
            'member_id' => $member_id,
            'same_day' => strtotime(date('Y-m-d')),
            'status' => 1
        );
        $log = p_get('shop_step_log', $where);
        if (empty($log)) {
            return true;
        }

        $shop_step = p_get('shop_step', array('member_id' => $member_id, 'same_day' => $today));
        $up = array('effective_step' => $log['step'] + $shop_step['effective_step']);
        p_update('shop_step', $up, array('id' => $shop_step['id']));
        p_update('shop_step_log', array('status' => 2), array('id' => $id));

        return true;
    }



    /**
     * @param $member_id
     * @param $step
     * @param int $type
     * @param string $sameday
     * @return bool
     */
    public function update_effective_step($member_id, $step, $type = 1, $sameday = '')
    {
        if (empty($sameday)) {
            $sameday = strtotime(date('Y-m-d'));
        }
        $shop_step = p_get('shop_step', array('member_id' => $member_id, 'same_day' => $sameday));

        $effective_step = $type == 1 ? ($shop_step['effective_step'] + $step) : ($shop_step['effective_step'] - $step);

        return p_update('shop_step', array('effective_step' => $effective_step), array('id' => $shop_step['id']));
    }



    /**
     * @param $member_id
     * @return int
     */
    public function get_signin($member_id)
    {
        $signin = p_get('shop_signin', array('member_id' => $member_id));
        if (empty($signin)) {
            return 0;
        }

        if ($signin['updated'] > strtotime(date('Y-m-d', time())) - 86400) {
            return $signin['days'];
        }
        return 0;
    }


    /**
     * @param $member_id
     * @return bool
     */
    public function today_signin($member_id)
    {
        $signin = p_get('shop_signin', array('member_id' => $member_id));
        if (empty($signin)) {
            return false;
        }
        $time = strtotime(date('Y-m-d', time()));
        return $signin['updated'] >= $time;
    }


    /**
     * @param $step
     * @return float
     */
    public function calculation_currency($step)
    {
        $step_currency = getConfig('step_currency');
        return round($step/$step_currency , 2);
    }


    /**
     * @param $member_id
     * @return bool
     */
    public function currency($member_id)
    {
        $step = $this->get_step($member_id);
        if (empty($step) || $step['effective_step'] < 1) {
            return true;
        }

        $currency = $this->calculation_currency($step['effective_step']);

        if ($currency <= 0.00) {
            return true;
        }

        if (!$this->change_currency($member_id, $currency, 2)) {
            return false;
        }
        return p_update('shop_step', array('effective_step' => 0), array('id' => $step['id']));
    }



    /**
     * @param $member_id
     * @return bool|int
     */
    public function signin($member_id)
    {
        try {
            pdo_begin();
            $sigin = p_get('shop_signin', array('member_id' => $member_id));

            if (empty($sigin)) {
                $insert = array(
                    'member_id' => $member_id,
                    'days' => 1,
                    'all_sigin' => 1,
                    'current_days' => 1,
                    'updated' => time(),
                    'created' => time()
                );
                if (!p_insert('shop_signin', $insert)) {
                    throw new Exception('签到失败,稍后再试');
                }
                $days = 1;
            } else {

                $today = strtotime(date('Y-m-d', time()));
                if ($sigin['updated'] >= $today) {
                    return $sigin['days'];
                }
                $update = array(
                    'all_sigin' => $sigin['all_sigin'] + 1,
                    'updated' => time()
                );
                if ($sigin['updated'] >= $today - 86400) {
                    $update['current_days'] = $sigin['current_days'] + 1;
                    $update['days'] = $sigin['days'] + 1;
                } else {
                    $update['current_days'] = 1;
                    $update['days'] = 1;
                }

                if (!p_update('shop_signin', $update, array('id' => $sigin['id']))) {
                    throw new Exception('签到失败,稍后再试');
                }
                $days = $update['days'];
            }

            if (!$this->update_effective_step($member_id, getConfig($this->signin_days[$days]['key']))) {
                throw new Exception('签到失败,稍后再试');
            }
            pdo_commit();
            return $days;
        } catch (Exception $e) {
            pdo_rollback();
            $this->error = $e->getMessage();
            return false;
        }
    }


    /**
     * @param $member_id
     * @param $treasure_id
     * @param $treasure_member_id
     * @param $number
     * @return bool
     */
    public function create_treasure_code($member_id, $treasure_id, $treasure_member_id, $number)
    {
        $data = array(
            'member_id' => $member_id,
            'treasure_id' => $treasure_id,
            'treasure_member_id' => $treasure_member_id,
            'created' => time()
        );
        for ($i = 0; $i < $number; $i++) {
            if (!p_insert('treasure_code', $data)) {
                $this->error = '生成夺宝码失败';
                return false;
            }
            $id = pdo_insertid();
            $no = $id . $treasure_id . $treasure_member_id;
            $content = str_pad($no, 10, '0', STR_PAD_LEFT);

            if (!p_update('treasure_code', array('code' => $content), array('id' => $id))) {
                $this->error = '生成夺宝码失败';
                return false;
            }
        }
        return true;
    }


    /**
     * @param $id
     * @return bool
     */
    public function openTreasure($id)
    {
        $treasure = p_get('treasure', array('id' => $id, 'is_delete' => 1, 'is_down' => 1, 'status' => 1));
        if (empty($treasure)) {
            $this->error = '改夺宝不存在或者已经结束';
            return false;
        }
        $code = p_get('treasure_code', array('treasure_id' => $id, 'is_appoint' => 2));
        if (empty($code)) {
            $sql = 'SELECT * FROM ' . prefix('treasure_code') . ' WHERE treasure_id=' . $id . ' ORDER BY RAND()';
            $code = p_fetch($sql);
        }

        pdo_begin();
        try {
            if (!p_update('treasure_code', array('in_prize' => 2), array('id' => $code['id']))) {
                throw new Exception('开奖失败');
            }
            $order = array(
                'member_id' => $code['member_id'],
                'treasure_id' => $id,
                'treasure_code_id' => $code['id'],
                'code' => $code['code'],
                'created' => time()
            );

            $address = p_get('address', array('member_id' => $code['member_id']));
            if (!empty($address)) {
                $order['name'] = $address['name'];
                $order['phone'] = $address['phone'];
                $order['address'] = $address['address'];
            }

            if (!p_insert('treasure_order', $order)) {
                throw new Exception('开奖失败');
            }
            if (!p_update('treasure', array('status' => 2, 'lottery_time' => time()), array('id' => $id))) {
                throw new Exception('开奖失败');
            }
            pdo_commit();
            return true;
        } catch (Exception $e) {
            pdo_rollback();
            $this->error = $e->getMessage();
            return false;
        }
    }



    /**
     * @param $member_id
     * @param $p
     * @param int $page_size
     * @return array
     */
    public function currency_log($member_id, $p, $page_size = 10)
    {
        $limit = ($p - 1) * $page_size;
        $log = p_getall('currency_log', array('member_id' => $member_id), array(),'', array('id desc'), "{$limit}, {$page_size}");

        if ($log) {
            foreach ($log as & $value) {
                if ($value['type'] == 10) {
                    $member = p_get('memberinfo', array('id' => $value['remarks']));
                    if ($member['nickname']) {
                        $value['remarks'] = '邀请奖励-' . $member['nickname'];
                    } else {
                        $value['remarks'] = '邀请奖励-游客' . substr($member['openid'], 0, 6);
                    }
                }
                $value['mold'] = in_array($value['type'], array(2, 5, 7, 8, 9, 10)) ? 1 : 2;
                $value['createTime'] = date('Y.m.d', $value['created']);
            }
        }

        return $log;
    }

}